"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Red Hat. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
const SchemaService = __importStar(require("../src/languageservice/services/yamlSchemaService"));
const url = __importStar(require("url"));
const sinon = __importStar(require("sinon"));
const chai = __importStar(require("chai"));
const sinon_chai_1 = __importDefault(require("sinon-chai"));
const expect = chai.expect;
chai.use(sinon_chai_1.default);
const workspaceContext = {
    resolveRelativePath: (relativePath, resource) => {
        return url.resolve(resource, relativePath);
    },
};
describe('YAML Schema', () => {
    const sandbox = sinon.createSandbox();
    let requestServiceStub;
    beforeEach(() => {
        requestServiceStub = sandbox.stub();
    });
    afterEach(() => {
        sandbox.restore();
    });
    it('Loading yaml scheme', async () => {
        requestServiceStub.resolves(`%YAML 1.2
---
properties:
  fooBar:
    items:
      type: string
    type: array
type: object
`);
        const service = new SchemaService.YAMLSchemaService(requestServiceStub, workspaceContext);
        const result = await service.loadSchema('fooScheme.yaml');
        expect(requestServiceStub.calledOnceWith('fooScheme.yaml'));
        expect(result.schema.properties['fooBar']).eql({
            items: { type: 'string' },
            type: 'array',
        });
    });
    it('Error while loading yaml', async () => {
        requestServiceStub.rejects();
        const service = new SchemaService.YAMLSchemaService(requestServiceStub, workspaceContext);
        const result = await service.loadSchema('fooScheme.yaml');
        expect(result.errors).length(1);
        expect(result.errors[0].message).includes('Unable to load schema from');
    });
    it('Error while parsing yaml scheme', async () => {
        requestServiceStub.resolves(`%464*&^^&*%@$&^##$`);
        const service = new SchemaService.YAMLSchemaService(requestServiceStub, workspaceContext);
        const result = await service.loadSchema('fooScheme.yaml');
        expect(requestServiceStub.calledOnceWith('fooScheme.yaml'));
        expect(result.errors).length(1);
        expect(result.errors[0].message).includes('Unable to parse content from');
    });
});
//# sourceMappingURL=yamlSchema.test.js.map