export type CustomTagInputType = 'mapping' | 'scalar' | 'sequence';
export type CustomTagReturnType = 'object' | 'array' | 'string' | 'number' | 'integer' | 'boolean' | 'null';
export interface CustomTag {
    tag: string;
    inputType: CustomTagInputType;
    returnType?: CustomTagReturnType;
}
export declare function parseCustomTag(customTagStr: unknown): CustomTag | undefined;
export declare function getCustomTagReturnType(node: unknown): CustomTagReturnType | undefined;
export declare function setCustomTagReturnType(node: unknown, returnType: CustomTagReturnType | undefined): void;
