import type { DocumentSymbol, SymbolInformation } from 'vscode-languageserver-types';
import type { DocumentSymbolsContext } from '../jsonLanguageTypes';
import type { TextDocument } from 'vscode-languageserver-textdocument';
import type { Telemetry } from '../telemetry';
export declare class YAMLDocumentSymbols {
    private readonly telemetry?;
    constructor(telemetry?: Telemetry);
    findDocumentSymbols(document: TextDocument, context?: DocumentSymbolsContext): SymbolInformation[];
    findHierarchicalDocumentSymbols(document: TextDocument, context?: DocumentSymbolsContext): DocumentSymbol[];
    private findFlatDocumentSymbols;
    private findHierarchicalSymbols;
}
