import { SingleYAMLDocument } from '../parser/yamlParser07';
import { JSONDocument } from '../parser/jsonDocument';
/**
 * Retrieve schema if declared by `$schema`.
 * Public for testing purpose, not part of the API.
 * @param doc
 */
export declare function getDollarSchema(doc: SingleYAMLDocument | JSONDocument): string | undefined;
